/*
 * Wacom W8001 Startup Utility for Android
 *
 * Copyright (c) 2011 Mike Bagdanoff, Wacom. <mikeb@wacom.com>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file COPYING in the main directory of this archive for
 * more details.
 *
 * 
 */

#include <linux/serio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <errno.h>

#define NORMAL_MODE_QRERY_LENGTH	11
#define NORMAL_MODE_QRERY_HEADER	0xC0



static void setline(int fd, int speed)
{
	struct termios t;

	tcgetattr(fd, &t);

	t.c_cflag = CS8 | CREAD | HUPCL | CLOCAL;
	t.c_iflag = IGNBRK | IGNPAR;
	t.c_oflag = 0;
	t.c_lflag = 0;
	t.c_cc[VMIN ] = 1;
	t.c_cc[VTIME] = 0;

	cfsetispeed(&t, speed);
	cfsetospeed(&t, speed);

	tcsetattr(fd, TCSANOW, &t);
}


int flash_query_normal(int fd)
{
	int rv, len;
	unsigned char buf[NORMAL_MODE_QRERY_LENGTH];
	unsigned char q_cmd;
	double tick, nTick;
	struct timeval tv;

	q_cmd = '0'; // Stop command
	rv = write(fd, &q_cmd, sizeof(q_cmd));
	if (rv <= 0)
		goto err;

	usleep(100*1000);

	q_cmd = '*'; //normal mode query command
	rv = write(fd, &q_cmd, sizeof(q_cmd));		/* Send querycommand */
	if (rv <= 0)
		goto err;

	gettimeofday(&tv, NULL);
	tick = tv.tv_sec + (double)tv.tv_usec*1e-6;
	while ( (read(fd, buf, 1) != 1 ) || (buf[0] != NORMAL_MODE_QRERY_HEADER)){
		gettimeofday(&tv, NULL);
		nTick = (tv.tv_sec + (double)tv.tv_usec*1e-6)-tick;
		if (nTick > 1.0){ /* Time out in 1 seconds */
			goto err;
		}
	}

	len = 1;

	gettimeofday(&tv, NULL);
	tick = tv.tv_sec + (double)tv.tv_usec*1e-6;
	do{
		rv = read(fd, buf, 1 );
		if(rv == 1){
			len++;
		}
		gettimeofday(&tv, NULL);
		nTick = (tv.tv_sec + (double)tv.tv_usec*1e-6)-tick;
		if (nTick > 1.0){ /* Time out in 1 seconds */
			goto err;
		}
	}while(len != NORMAL_MODE_QRERY_LENGTH);

	q_cmd = '1'; // 200Hz Scan Mode
	rv = write(fd, &q_cmd, sizeof(q_cmd));
	if (rv <= 0)
		goto err;

	usleep(100*1000);

	return 0;

err:
	return -1;
}


void startup(int fd)
{
	int ldisc = N_MOUSE;
	unsigned long devt = SERIO_W8001;

	if (ioctl(fd, TIOCSETD, &ldisc)) {
		fprintf(stderr, "wac_startup: can't set line discipline\n");
	}
	if (ioctl(fd, SPIOCSTYPE, &devt)) {
		fprintf(stderr, "wac_startup: can't set device type\n");
	}

	for ( ; ; ) {
		read(fd, NULL, 0);
	}

	ldisc = 0;
	ioctl(fd, TIOCSETD, &ldisc);
}


int main(int argc, char *argv[])
{
	static	int	baud[] = { B38400, B19200 };

	int fd;
	unsigned int index;

	if (argc < 2) {
		return 0;
	}

	fd = open(argv[1], O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (fd < 0) {
		fprintf(stderr, "wac_startup: '%s' - %s\n", argv[1], strerror(errno));
		return EXIT_FAILURE;
	}

	for (index = 0; index < (sizeof baud / sizeof baud[0]); index++) {
		setline(fd, baud[index]);
		if (flash_query_normal(fd) == 0) {
			startup(fd);
		}
	}

	close(fd);

	return EXIT_FAILURE;
}
